@@ -28,9 +28,12 @@
 #include "ContextMenu.h"
 #include "Piano.h"
 #include "Tools.h"
+#include "Tracker.h"
+#include "SampleEditor.h"
 #include "TrackerConfig.h"
 #include "PlayerController.h"
 #include "DialogBase.h"
+#include "FilterParameters.h"
 
 #include <algorithm>
 #include <math.h>
@@ -64,6 +67,7 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 										 EventListenerInterface* eventListener, 
 										 const PPPoint& location, 
 										 const PPSize& size, 
+										 Tracker& tracker,
 										 bool border/*= true*/) :
 	PPControl(id, parentScreen, eventListener, location, size),
 	border(border),
@@ -114,13 +118,32 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 	static const char* seperatorStringLarge = "\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4";
 	static const char* seperatorStringMed = "\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4\xc4";
 
-	subMenuAdvanced = new PPContextMenu(5, parentScreen, this, PPPoint(0,0), TrackerConfig::colorThemeMain);
+	subMenuFX = new PPContextMenu(6, parentScreen, this, PPPoint(0,0), TrackerConfig::colorPatternEditorCursorLine);
+	subMenuFX->addEntry("Volume" PPSTR_PERIODS, MenuCommandIDVolumeBoost);
+	subMenuFX->addEntry("Fade in" PPSTR_PERIODS, MenuCommandIDVolumeFadeIn);
+	subMenuFX->addEntry("Fade out" PPSTR_PERIODS, MenuCommandIDVolumeFadeOut);
+	subMenuFX->addEntry("Fade custom" PPSTR_PERIODS, MenuCommandIDVolumeFade);
+	subMenuFX->addEntry(seperatorStringLarge, -1);
+	subMenuFX->addEntry("Delay", MenuCommandIDDelay);
+	subMenuFX->addEntry("Reverb", MenuCommandIDReverb);
+	subMenuFX->addEntry("Vocoder", MenuCommandIDVocode);
+	subMenuFX->addEntry("Timestretch", MenuCommandIDTimeStretch);
+	subMenuFX->addEntry(seperatorStringLarge, -1);
+	subMenuFX->addEntry("Filter" PPSTR_PERIODS, MenuCommandIDFilter);
+	subMenuFX->addEntry("EQ  3 Band" PPSTR_PERIODS, MenuCommandIDEQ3Band);
+	subMenuFX->addEntry("EQ 10 Band" PPSTR_PERIODS, MenuCommandIDEQ10Band);
+	subMenuFX->addEntry(seperatorStringLarge, -1);
+	subMenuFX->addEntry("Compress", MenuCommandIDCompress);
+	subMenuFX->addEntry("Saturate", MenuCommandIDSaturate);
+	subMenuFX->addEntry("Excite #Milky", MenuCommandIDMTBoost);
+	subMenuFX->addEntry("Excite #PTboost", MenuCommandIDPTBoost);
+	subMenuFX->addEntry(seperatorStringLarge, -1);
+	subMenuFX->addEntry("Normalize", MenuCommandIDNormalize);
+	subMenuFX->addEntry("Backwards", MenuCommandIDReverse);
+	subMenuFX->addEntry("Loop fold" PPSTR_PERIODS, MenuCommandIDVolumeFold);
+
+	subMenuAdvanced = new PPContextMenu(5, parentScreen, this, PPPoint(0,0), TrackerConfig::colorPatternEditorCursorLine);
 	subMenuAdvanced->setSubMenu(true);
-	subMenuAdvanced->addEntry("Volume boost" PPSTR_PERIODS, MenuCommandIDVolumeBoost);
-	subMenuAdvanced->addEntry("Volume fade" PPSTR_PERIODS, MenuCommandIDVolumeFade);
-	subMenuAdvanced->addEntry("Normalize", MenuCommandIDNormalize);
-	subMenuAdvanced->addEntry(seperatorStringLarge, -1);
-	subMenuAdvanced->addEntry("Backwards", MenuCommandIDReverse);
 	subMenuAdvanced->addEntry("Cross-fade", MenuCommandIDXFade);
 	subMenuAdvanced->addEntry(seperatorStringLarge, -1);
 	subMenuAdvanced->addEntry("Change sign", MenuCommandIDChangeSign);
@@ -131,51 +154,59 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 	subMenuAdvanced->addEntry(seperatorStringLarge, -1);
 	subMenuAdvanced->addEntry("Smooth (rect.)", MenuCommandIDRectangularSmooth);
 	subMenuAdvanced->addEntry("Smooth (tri.)", MenuCommandIDTriangularSmooth);
-	subMenuAdvanced->addEntry("3 Band EQ" PPSTR_PERIODS, MenuCommandIDEQ3Band);
-	subMenuAdvanced->addEntry("10 Band EQ" PPSTR_PERIODS, MenuCommandIDEQ10Band);
 	subMenuAdvanced->addEntry(seperatorStringLarge, -1);
 	subMenuAdvanced->addEntry("Resample" PPSTR_PERIODS, MenuCommandIDResample);
 
-	subMenuXPaste = new PPContextMenu(8, parentScreen, this, PPPoint(0,0), TrackerConfig::colorThemeMain);
+	subMenuXPaste = new PPContextMenu(8, parentScreen, this, PPPoint(0,0), TrackerConfig::colorPatternEditorCursorLine);
 	subMenuXPaste->setSubMenu(true);
 	subMenuXPaste->addEntry("Mix", MenuCommandIDMixPaste);
-	subMenuXPaste->addEntry("Amp Modulate", MenuCommandIDAMPaste);
-	subMenuXPaste->addEntry("Freq Modulate", MenuCommandIDFMPaste);
-	subMenuXPaste->addEntry("Phase Modulate", MenuCommandIDPHPaste);
+	subMenuXPaste->addEntry("Mix (overflow)", MenuCommandIDMixOverflowPaste);
+	subMenuXPaste->addEntry("Mix (spread)", MenuCommandIDMixSpreadPaste);
+	subMenuXPaste->addEntry("Modulate Amp", MenuCommandIDAMPaste);
+	subMenuXPaste->addEntry("Modulate Freq", MenuCommandIDFMPaste);
+	subMenuXPaste->addEntry("Modulate Phase", MenuCommandIDPHPaste);
+	subMenuXPaste->addEntry("Flanger", MenuCommandIDFLPaste);
 	subMenuXPaste->addEntry("Selective EQ" PPSTR_PERIODS, MenuCommandIDSelectiveEQ10Band);
+	subMenuXPaste->addEntry("Capture pattern" PPSTR_PERIODS, MenuCommandIDCapturePattern);
 
-	subMenuPT = new PPContextMenu(6, parentScreen, this, PPPoint(0,0), TrackerConfig::colorThemeMain);
+
+	subMenuPT = new PPContextMenu(6, parentScreen, this, PPPoint(0,0), TrackerConfig::colorPatternEditorCursorLine);
 	subMenuPT->addEntry("Boost", MenuCommandIDPTBoost);
 
-	subMenuGenerators = new PPContextMenu(7, parentScreen, this, PPPoint(0,0), TrackerConfig::colorThemeMain);
+	subMenuGenerators = new PPContextMenu(7, parentScreen, this, PPPoint(0,0), TrackerConfig::colorPatternEditorCursorLine);
 	subMenuGenerators->addEntry("Noise" PPSTR_PERIODS, MenuCommandIDGenerateNoise);
 	subMenuGenerators->addEntry("Sine" PPSTR_PERIODS, MenuCommandIDGenerateSine);
 	subMenuGenerators->addEntry("Square" PPSTR_PERIODS, MenuCommandIDGenerateSquare);
 	subMenuGenerators->addEntry("Triangle" PPSTR_PERIODS, MenuCommandIDGenerateTriangle);
 	subMenuGenerators->addEntry("Sawtooth" PPSTR_PERIODS, MenuCommandIDGenerateSawtooth);
+	subMenuGenerators->addEntry("Half Sine" PPSTR_PERIODS, MenuCommandIDGenerateHalfSine);
+	subMenuGenerators->addEntry("Absolute Sine" PPSTR_PERIODS, MenuCommandIDGenerateAbsoluteSine);
+	subMenuGenerators->addEntry("Quarter Sine" PPSTR_PERIODS, MenuCommandIDGenerateQuarterSine);
 	subMenuGenerators->addEntry("Silence" PPSTR_PERIODS, MenuCommandIDGenerateSilence);
 	
 	// build context menu
-	editMenuControl = new PPContextMenu(4, parentScreen, this, PPPoint(0,0), TrackerConfig::colorThemeMain, true);
+	editMenuControl = new PPContextMenu(4, parentScreen, this, PPPoint(0,0), TrackerConfig::colorPatternEditorCursorLine, true);
 	editMenuControl->addEntry("New" PPSTR_PERIODS, MenuCommandIDNew);
 	editMenuControl->addEntry(seperatorStringMed, -1);
 	editMenuControl->addEntry("Undo", MenuCommandIDUndo);
 	editMenuControl->addEntry("Redo", MenuCommandIDRedo);
 	editMenuControl->addEntry(seperatorStringMed, -1);
+	editMenuControl->addEntry("FX           \x10", 0xFFFF, subMenuFX);
+	editMenuControl->addEntry("Generators   \x10", 0xFFFF, subMenuGenerators);
+	editMenuControl->addEntry("Paste        \x10", 0xFFFF, subMenuXPaste);
+	editMenuControl->addEntry("Advanced     \x10", 0xFFFF, subMenuAdvanced);
+	editMenuControl->addEntry(seperatorStringMed, -1);
 	editMenuControl->addEntry("Cut", MenuCommandIDCut);
 	editMenuControl->addEntry("Copy", MenuCommandIDCopy);
 	editMenuControl->addEntry("Paste", MenuCommandIDPaste);
 	editMenuControl->addEntry("Crop", MenuCommandIDCrop);
 	editMenuControl->addEntry("Range all", MenuCommandIDSelectAll);
-	editMenuControl->addEntry(seperatorStringMed, -1);
-	editMenuControl->addEntry("Advanced   \x10", 0xFFFF, subMenuAdvanced);
-	editMenuControl->addEntry("Ext. Paste \x10", 0xFFFF, subMenuXPaste);
-	editMenuControl->addEntry("Protracker \x10", 0xFFFF, subMenuPT);
-	editMenuControl->addEntry("Generators \x10", 0xFFFF, subMenuGenerators);
+	editMenuControl->addEntry("Loop range", MenuCommandIDLoopRange);
 
 	// Create tool handler responder
 	toolHandlerResponder = new ToolHandlerResponder(*this);
 	dialog = NULL;	
+	this->tracker = (Tracker *)&tracker;
 	
 	resetLastValues();
 }
@@ -196,6 +227,7 @@ SampleEditorControl::~SampleEditorControl()
 	delete editMenuControl;	
 	delete subMenuAdvanced;
 	delete subMenuXPaste;
+  delete subMenuFX;
 	delete subMenuPT;
 	delete subMenuGenerators;
 }
@@ -273,7 +305,7 @@ pp_uint32 SampleEditorControl::getRepeatLength() const
 	return sampleEditor->getRepeatLength(); 
 }
 
-void SampleEditorControl::formatMillis(char* buffer, pp_uint32 millis)
+void SampleEditorControl::formatMillis(char* buffer, size_t size, pp_uint32 millis)
 {
 	if (millis >= 1000)
 	{
@@ -283,16 +315,16 @@ void SampleEditorControl::formatMillis(char* buffer, pp_uint32 millis)
 		{
 			pp_uint32 mins = secs / 60;
 			secs %= 60;
-			sprintf(buffer, "%im%i.%is", mins, secs, millis);
+			snprintf(buffer, size, "%im%02i.%03is", mins, secs, millis);
 		}
 		else		
-			sprintf(buffer, "%i.%is", secs, millis);
+			snprintf(buffer, size, "%i.%03is", secs, millis);
 	}
 	else
-		sprintf(buffer, "%ims", millis);	
+		snprintf(buffer, size, "%ims", millis);
 }
 
-void SampleEditorControl::formatMillisFraction(char* buffer, pp_uint32 millis, pp_uint32 totalMillis)
+void SampleEditorControl::formatMillisFraction(char* buffer, size_t size, pp_uint32 millis, pp_uint32 totalMillis)
 {
 	if (totalMillis >= 1000)
 	{
@@ -306,13 +338,13 @@ void SampleEditorControl::formatMillisFraction(char* buffer, pp_uint32 millis, p
 			pp_uint32 totalMins = totalSecs / 60;
 			secs %= 60;
 			totalSecs %= 60;
-			sprintf(buffer, "%im%i.%is / %im%i.%is", mins, secs, millis, totalMins, totalSecs, totalMillis);
+			snprintf(buffer, size, "%im%02i.%03is / %im%02i.%03is", mins, secs, millis, totalMins, totalSecs, totalMillis);
 		}
 		else
-			sprintf(buffer, "%i.%is / %i.%is", secs, millis, totalSecs, totalMillis);
+			snprintf(buffer, size, "%i.%03is / %i.%03is", secs, millis, totalSecs, totalMillis);
 	}
 	else
-		sprintf(buffer, "%ims / %ims", millis, totalMillis);
+		snprintf(buffer, size, "%ims / %ims", millis, totalMillis);
 }
 
 void SampleEditorControl::paint(PPGraphicsAbstract* g)
@@ -512,7 +544,7 @@ void SampleEditorControl::paint(PPGraphicsAbstract* g)
 	else if (offsetFormat == OffsetFormatMillis)
 	{
 		pp_uint32 millis = sampleEditor->convertSmpPosToMillis((mp_sint32)ceil(startPos*xScale), relativeNote);
-		formatMillis(buffer, millis);
+		formatMillis(buffer, sizeof(buffer), millis);
 	}
 
 	PPFont* font = PPFont::getFont(PPFont::FONT_TINY);
@@ -537,7 +569,7 @@ void SampleEditorControl::paint(PPGraphicsAbstract* g)
 		{
 			pp_uint32 millis = sampleEditor->convertSmpPosToMillis(positionToSample(currentPosition), relativeNote);
 			pp_uint32 totalMillis = sampleEditor->convertSmpPosToMillis(end, relativeNote);
-			formatMillisFraction(buffer, millis, totalMillis);
+			formatMillisFraction(buffer, sizeof(buffer), millis, totalMillis);
 		}
 	}
 	else
@@ -549,7 +581,7 @@ void SampleEditorControl::paint(PPGraphicsAbstract* g)
 		else if (offsetFormat == OffsetFormatMillis)
 		{
 			pp_uint32 totalMillis = sampleEditor->convertSmpPosToMillis(end, relativeNote);
-			formatMillis(buffer, totalMillis);
+			formatMillis(buffer, sizeof(buffer), totalMillis);
 		}
 	}
 
@@ -1096,16 +1128,16 @@ selectingAndResizing:
 					pp_int32 x2 = (pp_int32)((sEnd)/xScale)-startPos + location.x + 2;
 					
 					pp_int32 minDist = (scrollDist>>4);
-					if (minDist < 1) minDist = 1;
+					if (minDist < 4) minDist = 4;
 					
 					pp_int32 sDist1 = abs(x1 - p->x);
 					pp_int32 sDist2 = abs(p->x - x2);
 					
-					if (sDist1 >= 0 && sDist1 <= minDist && (::getKeyModifier() == KeyModifierCTRL))
+					if (sDist1 >= 0 && sDist1 <= minDist)
 					{
 						type = MouseCursorTypeResizeLeft;
 					}
-					else if (sDist2 >= 0 && sDist2 <= minDist && (::getKeyModifier() == KeyModifierCTRL))
+					else if (sDist2 >= 0 && sDist2 <= minDist)
 					{
 						type = MouseCursorTypeResizeRight;
 					}
@@ -1198,6 +1230,7 @@ pp_int32 SampleEditorControl::handleEvent(PPObject* sender, PPEvent* event)
 			 sender == reinterpret_cast<PPObject*>(subMenuAdvanced) ||
 			 sender == reinterpret_cast<PPObject*>(subMenuXPaste) ||
 			 sender == reinterpret_cast<PPObject*>(subMenuPT) ||
+			 sender == reinterpret_cast<PPObject*>(subMenuFX) ||
 			 sender == reinterpret_cast<PPObject*>(subMenuGenerators)) &&
 			 event->getID() == eCommand)
 	{
@@ -1436,6 +1469,14 @@ void SampleEditorControl::rangeAll(bool updateNotify/* = false*/)
 		notifyUpdate();
 }
 
+void SampleEditorControl::loopRange(bool updateNotify/* = false*/)
+{	
+	sampleEditor->loopRange();
+
+	if (updateNotify)
+		notifyUpdate();
+}
+
 void SampleEditorControl::rangeClear(bool updateNotify/* = false*/)
 {
 	sampleEditor->resetSelection();
@@ -1663,10 +1704,17 @@ void SampleEditorControl::invokeContextMenu(const PPPoint& p, bool translatePoin
 	editMenuControl->setState(MenuCommandIDCut, !hasValidSelection());
 	editMenuControl->setState(MenuCommandIDCrop, !hasValidSelection());
 	editMenuControl->setState(MenuCommandIDSelectAll, isEmptySample);
+	editMenuControl->setState(MenuCommandIDLoopRange, !hasValidSelection());
 	
 	// update submenu states
 	subMenuAdvanced->setState(MenuCommandIDNormalize, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDCompress, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDMTBoost, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDReverb, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDVolumeFade, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDVolumeFadeIn, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDVolumeFadeOut, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDVolumeFold, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDVolumeBoost, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDReverse, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDXFade, !sampleEditor->isValidxFadeSelection() || isEmptySample || !sampleEditor->getLoopType());
@@ -1676,25 +1724,35 @@ void SampleEditorControl::invokeContextMenu(const PPPoint& p, bool translatePoin
 	subMenuAdvanced->setState(MenuCommandIDDCOffset, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDRectangularSmooth, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDTriangularSmooth, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDFilter, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDEQ3Band, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDEQ10Band, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDResample, isEmptySample);
 
 	subMenuXPaste->setState(MenuCommandIDMixPaste, sampleEditor->clipBoardIsEmpty() || isEmptySample);
+	subMenuXPaste->setState(MenuCommandIDMixOverflowPaste, sampleEditor->clipBoardIsEmpty() || isEmptySample);
+	subMenuXPaste->setState(MenuCommandIDMixSpreadPaste, sampleEditor->clipBoardIsEmpty() || isEmptySample);
 	subMenuXPaste->setState(MenuCommandIDAMPaste, sampleEditor->clipBoardIsEmpty() || isEmptySample);
 	subMenuXPaste->setState(MenuCommandIDFMPaste, sampleEditor->clipBoardIsEmpty() || isEmptySample);
 	subMenuXPaste->setState(MenuCommandIDPHPaste, sampleEditor->clipBoardIsEmpty() || isEmptySample);
+	subMenuXPaste->setState(MenuCommandIDFLPaste, sampleEditor->clipBoardIsEmpty() || isEmptySample);
 	subMenuXPaste->setState(MenuCommandIDSelectiveEQ10Band, sampleEditor->clipBoardIsEmpty() || isEmptySample);
 
 	subMenuPT->setState(MenuCommandIDPTBoost, isEmptySample);
+	subMenuPT->setState(MenuCommandIDSaturate, isEmptySample);
+	subMenuPT->setState(MenuCommandIDTimeStretch, isEmptySample);
+	subMenuPT->setState(MenuCommandIDDelay, isEmptySample);
 
 	subMenuGenerators->setState(MenuCommandIDGenerateNoise, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateSine, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateSquare, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateTriangle, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateSawtooth, isEmptySample);
+	subMenuGenerators->setState(MenuCommandIDGenerateHalfSine, isEmptySample);
+	subMenuGenerators->setState(MenuCommandIDGenerateAbsoluteSine, isEmptySample);
+	subMenuGenerators->setState(MenuCommandIDGenerateQuarterSine, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateSilence, isEmptySample);
-
+	
 	parentScreen->setContextMenuControl(editMenuControl);
 }
 
@@ -1706,6 +1764,7 @@ void SampleEditorControl::hideContextMenu()
 
 void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 {
+
 	switch (commandId)
 	{
 		// cut
@@ -1723,7 +1782,17 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			sampleEditor->paste();
 			break;
 
-		// mix-paste
+		// mix-paste spread
+		case MenuCommandIDMixOverflowPaste:
+			sampleEditor->mixOverflowPasteSample();
+			break;
+
+		// mix-paste spread
+		case MenuCommandIDMixSpreadPaste:
+			sampleEditor->mixSpreadPasteSample();
+			break;
+
+		// mix-paste stencil-like (preserves pitch)
 		case MenuCommandIDMixPaste:
 			sampleEditor->mixPasteSample();
 			break;
@@ -1743,6 +1812,11 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			sampleEditor->PHPasteSample();
 			break;
 
+		// FL-paste
+		case MenuCommandIDFLPaste:
+			sampleEditor->FLPasteSample();
+			break;
+
 		// crop
 		case MenuCommandIDCrop:
 			sampleEditor->cropSample();
@@ -1763,11 +1837,20 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			rangeAll(true);
 			break;
 					
+		// set loop to current selection
+		case MenuCommandIDLoopRange:
+			loopRange(true);
+			break;	
+
 		// Invoke tools
 		case MenuCommandIDNew:
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeNew);
 			break;
 
+		case MenuCommandIDCapturePattern:
+			tracker->eventKeyDownBinding_InvokePatternCapture();
+			break;
+
 		case MenuCommandIDVolumeBoost:
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeVolume);
 			break;
@@ -1776,6 +1859,27 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeFade);
 			break;
 
+		case MenuCommandIDVolumeFadeOut:{
+			FilterParameters par(2);
+			par.setParameter(0, FilterParameters::Parameter(100.0f));
+			par.setParameter(1, FilterParameters::Parameter(0.0f));
+			sampleEditor->tool_scaleSample(&par);
+			break;
+		}
+
+		case MenuCommandIDVolumeFadeIn:{
+			FilterParameters par(2);
+			par.setParameter(0, FilterParameters::Parameter(0.0f));
+			par.setParameter(1, FilterParameters::Parameter(100.0f));
+			sampleEditor->tool_scaleSample(&par);
+			break;
+		}
+
+		case MenuCommandIDVolumeFold:{
+			sampleEditor->tool_foldSample(NULL);
+			break;
+		}
+
 		case MenuCommandIDResample:
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeResample);
 			break;
@@ -1788,6 +1892,22 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			sampleEditor->tool_normalizeSample(NULL);
 			break;
 
+		case MenuCommandIDCompress:
+			sampleEditor->tool_compressSample(NULL);
+			break;
+
+		case MenuCommandIDMTBoost:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeMTBoost);
+			break;
+
+		case MenuCommandIDVocode:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeVocode);
+			break;
+
+		case MenuCommandIDReverb:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeReverb);
+			break;
+
 		case MenuCommandIDReverse:
 			sampleEditor->tool_reverseSample(NULL);
 			break;
@@ -1796,6 +1916,18 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			sampleEditor->tool_PTboostSample(NULL);
 			break;
 
+		case MenuCommandIDSaturate:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeSaturate);
+			break;
+
+		case MenuCommandIDTimeStretch:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeTimeStretch);
+			break;
+
+		case MenuCommandIDDelay:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeDelay);
+			break;
+
 		case MenuCommandIDXFade:
 			sampleEditor->tool_xFadeSample(NULL);
 			break;
@@ -1820,7 +1952,11 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			sampleEditor->tool_triangularSmoothSample(NULL);
 			break;
 
-		case MenuCommandIDEQ3Band:
+		case MenuCommandIDFilter:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeFilter);
+			break;
+		
+    case MenuCommandIDEQ3Band:
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeEQ3Band);
 			break;
 
@@ -1856,6 +1992,21 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateSawtooth);
 			break;
 
+		case MenuCommandIDGenerateHalfSine:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateHalfSine);
+			break;
+
+		case MenuCommandIDGenerateAbsoluteSine:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateAbsoluteSine);
+			break;
+
+		case MenuCommandIDGenerateQuarterSine:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateQuarterSine);
+			break;
+
+		case MenuCommandIDSynth:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeSynth);
+			break;
 	}
 }
 
